@@ -23,6 +23,20 @@ from .config import Config
 
 VERBOSE_REFRESH_TIME=1.0  # 1 second
 
+def is_hdf5(domain, **kwargs):
+    """ Determine if domain is valid HSDS domain. 
+    kwargs can be endpoint, username, password, etc. (same as with File) 
+    """
+    found = False
+    try:
+        # set use_cache to False to avoid extensive load time
+        f = File(domain, use_cache=False, **kwargs)
+        found = True
+    except IOError:
+        pass # ignore any non-200 error
+    return found
+ 
+
 
 class File(Group):
 
@@ -194,45 +208,6 @@ class File(Group):
                 elif "hs_bucket" in cfg:
                     bucket = cfg["hs_bucket"]
 
-            if api_key is None:
-                if "HS_API_KEY" in os.environ:
-                    api_key = os.environ["HS_API_KEY"]
-                elif "hs_api_key" in cfg:
-                    api_key = cfg["hs_api_key"]
-
-            if not api_key:
-                # if Azure AD ids are set, pass them to HttpConn via api_key dict
-
-                ad_app_id = None  # Azure AD HSDS Server id
-                if "HS_AD_APP_ID" in os.environ:
-                    ad_app_id = os.environ["HS_AD_APP_ID"]
-                elif "hs_ad_app_id" in cfg:
-                    ad_app_id = cfg["hs_ad_app_id"]
-
-                ad_tenant_id = None # Azure AD tenant id
-                if "HS_AD_TENANT_ID" in os.environ:
-                    ad_tenant_id = os.environ["HS_AD_TENANT_ID"]
-                elif "hs_ad_tenant_id" in cfg:
-                    ad_tenant_id = cfg["hs_ad_tenant_id"]
-
-                ad_resource_id = None # Azure AD resource id
-                if "HS_AD_RESOURCE_ID" in os.environ:
-                    ad_resource_id = os.environ["HS_AD_RESOURCE_ID"]
-                elif "hs_ad_resource_id" in cfg:
-                    ad_resource_id = cfg["hs_ad_resource_id"]
-
-                ad_client_secret = None # Azure client secret
-                if "HS_AD_CLIENT_SECRET" in os.environ:
-                    ad_client_secret = os.environ["HS_AD_CLIENT_SECRET"]
-                elif "hs_ad_client_secret" in cfg:
-                    ad_client_secret = cfg["hs_ad_client_secret"]
-
-                if ad_app_id and ad_tenant_id and ad_resource_id:
-                    # contruct dict to pass to HttpConn
-                    api_key = {"AD_APP_ID": ad_app_id, "AD_TENANT_ID": ad_tenant_id, "AD_RESOURCE_ID": ad_resource_id}
-                    if ad_client_secret:
-                        api_key["AD_CLIENT_SECRET"] = ad_client_secret
-
             http_conn =  HttpConn(domain, endpoint=endpoint,
                     username=username, password=password, bucket=bucket, mode=mode,
                     api_key=api_key, use_session=use_session, use_cache=use_cache, logger=logger, retries=retries)
